@@ -117,11 +117,8 @@ public:
 };
 
 
-vcd_trace::vcd_trace(const std::string& name_,
-		     const std::string& vcd_name_)
-: name(name_),
-  vcd_name(vcd_name_),
-  bit_width(0)
+vcd_trace::vcd_trace(const std::string& name_, const std::string& vcd_name_)
+: name(name_), vcd_name(vcd_name_), vcd_var_typ_name(0), bit_width(0)
 {
     /* Intentionally blank */
 }
@@ -169,7 +166,7 @@ vcd_trace::print_variable_declaration_line(FILE* f)
     if ( bit_width <= 0 )
     {
         std::sprintf(buf, "Traced object \"%s\" has 0 Bits, cannot be traced.",
-	    name.c_str());
+	             name.c_str());
         put_error_message(buf, false);
     }
     else
@@ -179,19 +176,19 @@ vcd_trace::print_variable_declaration_line(FILE* f)
 	if ( bit_width == 1 )
 	{
 	    std::sprintf(buf, "$var %s  % 3d  %s  %s       $end\n",
-		vcd_var_typ_name,
-		bit_width,
-		vcd_name.c_str(),
-		namecopy.c_str());
+		         vcd_var_typ_name,
+		         bit_width,
+		         vcd_name.c_str(),
+		         namecopy.c_str());
 	}
 	else
 	{
 	    std::sprintf(buf, "$var %s  % 3d  %s  %s [%d:0]  $end\n",
-                vcd_var_typ_name,
-		bit_width,
-		vcd_name.c_str(),
-		namecopy.c_str(),
-		bit_width-1);
+                         vcd_var_typ_name,
+		         bit_width,
+		         vcd_name.c_str(),
+		         namecopy.c_str(),
+		         bit_width-1);
 	}
         std::fputs(buf, f);
     }
@@ -313,11 +310,10 @@ protected:
 vcd_bool_trace::vcd_bool_trace(const bool& object_,
 			       const std::string& name_,
 			       const std::string& vcd_name_)
-: vcd_trace(name_, vcd_name_), object(object_)
+: vcd_trace(name_, vcd_name_), object(object_), old_value(object_)
 {
     vcd_var_typ_name = "wire";
     bit_width = 1;
-    old_value = object;
 }
 
 bool
@@ -354,11 +350,10 @@ protected:
 vcd_sc_bit_trace::vcd_sc_bit_trace( const sc_dt::sc_bit& object_,
 				    const std::string& name,
 				    const std::string& vcd_name)
-: vcd_trace(name, vcd_name), object( object_ )
+: vcd_trace(name, vcd_name), object( object_ ), old_value( object_ )
 {
     vcd_var_typ_name = "wire";
     bit_width = 1;
-    old_value = object;
 }
 
 bool
@@ -397,11 +392,10 @@ protected:
 vcd_sc_logic_trace::vcd_sc_logic_trace(const sc_dt::sc_logic& object_,
 				       const std::string& name_,
 				       const std::string& vcd_name_) 
-: vcd_trace(name_, vcd_name_), object(object_)
+: vcd_trace(name_, vcd_name_), object(object_), old_value(object_)
 {
     vcd_var_typ_name = "wire";
     bit_width = 1;
-    old_value = object;
 }
 
 
@@ -674,11 +668,10 @@ vcd_sc_fxval_trace::vcd_sc_fxval_trace( const sc_dt::sc_fxval& object_,
 				        const std::string& name_,
 					const std::string& vcd_name_ )
 : vcd_trace( name_, vcd_name_ ),
-  object( object_ )
+  object( object_ ), old_value( object_ )
 {
     vcd_var_typ_name = "real";
     bit_width = 1;
-    old_value = object;
 }
 
 bool
@@ -718,11 +711,10 @@ vcd_sc_fxval_fast_trace::vcd_sc_fxval_fast_trace(
 					const std::string& name_,
 					const std::string& vcd_name_ )
 : vcd_trace( name_, vcd_name_ ),
-  object( object_ )
+  object( object_ ), old_value( object_ )
 {
     vcd_var_typ_name = "real";
     bit_width = 1;
-    old_value = object;
 }
 
 bool
@@ -891,17 +883,13 @@ vcd_unsigned_int_trace::vcd_unsigned_int_trace(
 					    const std::string& name_,
 					    const std::string& vcd_name_,
 					    int width_) 
-: vcd_trace(name_, vcd_name_), object(object_)
+: vcd_trace(name_, vcd_name_), object(object_), old_value( object_ ),
+  mask((unsigned)-1)
 {
     bit_width = width_;
-    if (bit_width < 32) {
-        mask = ~(-1 << bit_width);
-    } else {
-        mask = 0xffffffff;
-    }
+    if (bit_width < 32) mask = ~(-1 << bit_width);
 
     vcd_var_typ_name = "wire";
-    old_value = object;
 }
 
 
@@ -961,17 +949,12 @@ vcd_unsigned_short_trace::vcd_unsigned_short_trace(
 					const std::string& name_,
 					const std::string& vcd_name_,
 					int width_) 
-: vcd_trace(name_, vcd_name_), object(object_)
+: vcd_trace(name_, vcd_name_), object(object_), old_value(object_), mask(0xffff)
 {
     bit_width = width_;
-    if (bit_width < 16) {
-        mask = ~(-1 << bit_width);
-    } else {
-        mask = 0xffff;
-    }
+    if (bit_width < 16) mask = ~(-1 << bit_width);
 
     vcd_var_typ_name = "wire";
-    old_value = object;
 }
 
 
@@ -1031,17 +1014,11 @@ vcd_unsigned_char_trace::vcd_unsigned_char_trace(
 				const std::string& name_,
 				const std::string& vcd_name_,
 				int width_)
-: vcd_trace(name_, vcd_name_), object(object_)
+: vcd_trace(name_, vcd_name_), object(object_), old_value(object_), mask(0xff)
 {
     bit_width = width_;
-    if (bit_width < 8) {
-        mask = ~(-1 << bit_width);
-    } else {
-        mask = 0xff;
-    }
-
+    if (bit_width < 8) mask = ~(-1 << bit_width);
     vcd_var_typ_name = "wire";
-    old_value = object;
 }
 
 
@@ -1099,17 +1076,14 @@ vcd_unsigned_long_trace::vcd_unsigned_long_trace(
 				const std::string& name_,
 				const std::string& vcd_name_,
 				int width_)
-: vcd_trace(name_, vcd_name_), object(object_)
+: vcd_trace(name_, vcd_name_), object(object_), old_value(object_), 
+  mask((unsigned long)-1)
 {
     bit_width = width_;
-    if (bit_width < 32) {
-        mask = ~(-1 << bit_width);
-    } else {
-        mask = 0xffffffff;
-    }
+    if ( bit_width < (int)(sizeof(unsigned long)*8) ) 
+        mask = ~(-1L << bit_width);
 
     vcd_var_typ_name = "wire";
-    old_value = object;
 }
 
 
@@ -1132,7 +1106,7 @@ void vcd_unsigned_long_trace::write(FILE* f)
         }
     }
     else{
-        unsigned bit_mask = 1 << (bit_width-1);
+        unsigned long bit_mask = 1ul << (bit_width-1);
         for (bitindex = 0; bitindex < bit_width; bitindex++) {
             rawdata[bitindex] = (object & bit_mask)? '1' : '0';
             bit_mask = bit_mask >> 1;
@@ -1166,17 +1140,13 @@ vcd_signed_int_trace::vcd_signed_int_trace(const signed& object_,
 					   const std::string& name_,
 					   const std::string& vcd_name_,
 					   int width_)
-: vcd_trace(name_, vcd_name_), object(object_)
+: vcd_trace(name_, vcd_name_), object(object_), old_value(object_),  
+  mask((unsigned)-1)
 {
     bit_width = width_;
-    if (bit_width < 32) {
-        mask = ~(-1 << bit_width);
-    } else {
-        mask = 0xffffffff;
-    }
+    if (bit_width < 32) mask = ~(-1 << bit_width);
 
     vcd_var_typ_name = "wire";
-    old_value = object;
 }
 
 
@@ -1234,17 +1204,12 @@ vcd_signed_short_trace::vcd_signed_short_trace(
 					const std::string& name_,
 					const std::string& vcd_name_,
 					int width_)
-: vcd_trace(name_, vcd_name_), object(object_)
+: vcd_trace(name_, vcd_name_), object(object_), old_value(object_), mask(0xffff)
 {
     bit_width = width_;
-    if (bit_width < 16) {
-        mask = ~(-1 << bit_width);
-    } else {
-        mask = 0xffff;
-    }
+    if (bit_width < 16) mask = ~(-1 << bit_width);
 
     vcd_var_typ_name = "wire";
-    old_value = object;
 }
 
 
@@ -1301,17 +1266,12 @@ vcd_signed_char_trace::vcd_signed_char_trace(const char& object_,
 					     const std::string& name_,
 					     const std::string& vcd_name_,
 					     int width_)
-: vcd_trace(name_, vcd_name_), object(object_)
+: vcd_trace(name_, vcd_name_), object(object_), old_value(object_), mask(0xff)
 {
     bit_width = width_;
-    if (bit_width < 8) {
-        mask = ~(-1 << bit_width);
-    } else {
-        mask = 0xff;
-    }
+    if (bit_width < 8) mask = ~(-1 << bit_width);
 
     vcd_var_typ_name = "wire";
-    old_value = object;
 }
 
 
@@ -1368,14 +1328,12 @@ vcd_int64_trace::vcd_int64_trace(const sc_dt::int64& object_,
 					     const std::string& name_,
 					     const std::string& vcd_name_,
 					     int width_)
-: vcd_trace(name_, vcd_name_), object(object_)
+: vcd_trace(name_, vcd_name_), object(object_), old_value(object_), 
+  mask((sc_dt::uint64)-1)
 {
     bit_width = width_;
-	mask = (sc_dt::uint64)-1;
     if (bit_width < 64)  mask = ~(mask << bit_width);
-
     vcd_var_typ_name = "wire";
-    old_value = object;
 }
 
 
@@ -1437,14 +1395,12 @@ vcd_uint64_trace::vcd_uint64_trace(  const sc_dt::uint64& object_,
 				     const std::string& name_,
 				     const std::string& vcd_name_,
 				     int width_)
-: vcd_trace(name_, vcd_name_), object(object_)
+: vcd_trace(name_, vcd_name_), object(object_), old_value(object_),
+  mask((sc_dt::uint64)-1)
 {
     bit_width = width_;
-	mask = (sc_dt::uint64)-1;
-	if ( bit_width < 64 ) mask = ~(mask << bit_width);
-
+    if ( bit_width < 64 ) mask = ~(mask << bit_width);
     vcd_var_typ_name = "wire";
-    old_value = object;
 }
 
 
@@ -1506,17 +1462,13 @@ vcd_signed_long_trace::vcd_signed_long_trace(const long& object_,
 					     const std::string& name_,
 					     const std::string& vcd_name_,
 					     int width_)
-: vcd_trace(name_, vcd_name_), object(object_)
+: vcd_trace(name_, vcd_name_), object(object_), old_value(object_),
+  mask((unsigned long)-1)
 {
     bit_width = width_;
-    if (bit_width < 32) {
-        mask = ~(-1 << bit_width);
-    } else {
-        mask = 0xffffffff;
-    }
-
+    if ( bit_width < (int)(sizeof(long)*8) ) 
+        mask = ~(-1L << bit_width);
     vcd_var_typ_name = "wire";
-    old_value = object;
 }
 
 
@@ -1539,7 +1491,7 @@ void vcd_signed_long_trace::write(FILE* f)
         }
     }
     else{
-        unsigned bit_mask = 1 << (bit_width-1);
+        unsigned long bit_mask = 1ul << (bit_width-1);
         for (bitindex = 0; bitindex < bit_width; bitindex++) {
             rawdata[bitindex] = (object & bit_mask)? '1' : '0';
             bit_mask = bit_mask >> 1;
@@ -1606,11 +1558,10 @@ protected:
 vcd_double_trace::vcd_double_trace(const double& object_,
 				   const std::string& name_,
 				   const std::string& vcd_name_)
-: vcd_trace(name_, vcd_name_), object(object_)
+: vcd_trace(name_, vcd_name_), object(object_), old_value(object_)
 {
     vcd_var_typ_name = "real";
     bit_width = 1;
-    old_value = object;
 }
 
 bool vcd_double_trace::changed()
@@ -1649,10 +1600,11 @@ vcd_enum_trace::vcd_enum_trace(const unsigned& object_,
 			       const std::string& name_,
 			       const std::string& vcd_name_,
 			       const char** enum_literals_)
-: vcd_trace(name_, vcd_name_), object(object_), literals(enum_literals_)
+: vcd_trace(name_, vcd_name_), object(object_), old_value(object_),
+  mask(0xffffffff), literals(enum_literals_), nliterals(0)
 {
     // find number of bits required to represent enumeration literal - counting loop
-    for (nliterals = 0; enum_literals_[nliterals]; nliterals++);
+    for (nliterals = 0; enum_literals_[nliterals]; nliterals++) continue;
 
     // Figure out number of bits required to represent the number of literals
     bit_width = 0;
@@ -1670,7 +1622,6 @@ vcd_enum_trace::vcd_enum_trace(const unsigned& object_,
     }
 
     vcd_var_typ_name = "wire";
-    old_value = object;
 }       
 
 bool vcd_enum_trace::changed()
@@ -1690,7 +1641,7 @@ void vcd_enum_trace::write(FILE* f)
             rawdata[bitindex] = 'x';
         }
     } else {
-        unsigned bit_mask = 1 << (bit_width-1);
+        unsigned long bit_mask = 1ul << (bit_width-1);
         for (bitindex = 0; bitindex < bit_width; bitindex++) {
             rawdata[bitindex] = (object & bit_mask)? '1' : '0';
             bit_mask = bit_mask >> 1;
@@ -1708,6 +1659,10 @@ void vcd_enum_trace::write(FILE* f)
  *****************************************************************************/
 
 vcd_trace_file::vcd_trace_file(const char *name)
+: fp(0), trace_delta_cycles(false), vcd_name_index(0),
+  previous_time_units_low(0), previous_time_units_high(0), traces(),
+  initialized(false), timescale_unit(sc_get_time_resolution().to_seconds()),
+  timescale_set_by_user(false)
 {
     std::string file_name = name ;
     file_name += ".vcd";
@@ -2016,8 +1971,8 @@ vcd_trace_file::cycle(bool this_is_a_delta_cycle)
     double_to_special_int64(now_units, &now_units_high, &now_units_low );
 
     bool now_later_than_previous_time = false;
-    if( now_units_low > previous_time_units_low 
-        && now_units_high == previous_time_units_high
+    if( (now_units_low > previous_time_units_low 
+        && now_units_high == previous_time_units_high)
         || now_units_high > previous_time_units_high){
         now_later_than_previous_time = true;
     }
@@ -2164,19 +2119,28 @@ vcd_trace_file::obtain_name()
     const int used_types_count = 'z' - 'a' + 1;
     int result;
 
-    char char4 = (char)(vcd_name_index % used_types_count);
+    result = vcd_name_index;
+    char char6 = (char)(vcd_name_index % used_types_count);
 
-    result = vcd_name_index / used_types_count;
+    result = result / used_types_count;
+    char char5 = (char)(result % used_types_count);
+
+    result = result / used_types_count;
+    char char4 = (char)(result % used_types_count);
+
+    result = result / used_types_count;
     char char3 = (char)(result % used_types_count);
 
     result = result / used_types_count;
     char char2 = (char)(result % used_types_count);
 
     char buf[20];
-    std::sprintf(buf, "%c%c%c",
+    std::sprintf(buf, "%c%c%c%c%c",
             char2 + first_type_used,
             char3 + first_type_used,
-            char4 + first_type_used);
+            char4 + first_type_used,
+            char5 + first_type_used,
+            char6 + first_type_used);
     vcd_name_index++;
     return std::string(buf);
 }
@@ -2256,7 +2220,46 @@ remove_vcd_name_problems(std::string& name)
     }
 }
 
+// set the time unit.
+void vcd_trace_file::set_time_unit(double v, sc_time_unit tu)
+{
+    if(initialized)
+    {        
+	put_error_message(
+	    "Trace timescale unit cannot be changed once tracing has begun.",
+	    false
+        );
+        std::cout << "To change the scale, create a new trace file." 
+		  << std::endl;
+        return;    
+    }
 
+    switch ( tu )
+    {
+      case SC_FS:  v = v * 1e-15; break;
+	  case SC_PS:  v = v * 1e-12; break;
+      case SC_NS:  v = v * 1e-9;  break;
+      case SC_US:  v = v * 1e-6;  break;
+      case SC_MS:  v = v * 1e-3;  break;
+      case SC_SEC:                break;
+      default:                    
+	  	put_error_message("Unknown time unit specified ",true);
+		std::cout << tu << std::endl;
+		break;
+    }
+
+    timescale_unit = v;
+
+    // EMIT ADVISORY MESSAGE ABOUT CHANGE IN TIME SCALE:
+
+    char buf[200];
+    std::sprintf(buf,
+        "Note: VCD trace timescale unit is set by user to %e sec.\n",
+        timescale_unit);
+    ::std::cout << buf << ::std::flush;
+
+	timescale_set_by_user = true;
+}
 sc_trace_file*
 sc_create_vcd_trace_file(const char * name)
 {
